#********************************************************************************
# qware build environment - qube                                                *
#                                                                               *
#  Author: Kai Peter ©2015-YYYY                                                 *
# Version: QVERSION                                                             *
# License: ISC                                                                  *
#                                                                               *
# Description: The main control program.                                        *
#********************************************************************************
#---------- begin (standard) qware configuration section ------------------------
VERSION=`cat $0 | grep -m 1 'Version' | cut -d\   -f3`       # get version number
ME=`basename $0`
[ $DBG ] ||  DBG=0                        # initialize debug level (default: off)

version() { echo "$ME-$VERSION"; exit 0; }

LIBDIR="QLIBDIR"
error() { printf "\033[1m%s\033[0m\n" "$1" ; exit 1; }

help() {
  echo "Usage: qube <module>"
  echo
  echo "Modules:"
  printf "\tchglog\t update the CHANGELOG to latest git commits\n"
  printf "\tconfig\t edit the 'config' file\n"
  printf  "\tedit  \t edit a file in dir .qube, requires argument <file>\n"
  printf "\tinit  \t initialize a project\n"
  printf "\tsrcpkg\t create a distributable source code tarball\n"
  echo
  echo "Options:"
  printf "     -h|--help\t print this help\n"
  printf "     --version\t print version\n"
  exit $?
}
# Calling the next functions is slighly different like used in qube-* files, thus
# we keep it here hard-coded. Used with doConfig or doEdit here only!
chkCfgFile() {
  CFGFILE=".qube/config"
  [ ! -f "$CFGFILE" ] && error "No config file found!" || source "$CFGFILE"
}
getTopQubeDir() {             # top level directory where we start any processing
  CURDIR=`pwd`
  until [ -d .qube ] ; do
    cd ..
    [ `pwd` = "/" ] && error "Not a qube project up to dir '/'."
    df | grep -q `pwd` && error "Not a qube project up to mountpoint `pwd`."
  done
  TOPQUBEDIR=`pwd`
  chkCfgFile
}

doConfig() { getTopQubeDir; $EDITOR .qube/config ; exit $?; }
doEdit() {
  [ ! "$1" ] && error "Edit requires an file as argument."
  getTopQubeDir
error "Not implemented yet."
  return
}

doBinPkg() { echo "Not implemented yet." ; exit 1 ; }
doChgLog() { "$LIBDIR/qube-chglog" ; exit $? ; }
doSrcPkg() { "$LIBDIR/qube-srcpkg" ; exit $? ; }
doUpLoad() { echo "Not implemented yet." ; exit 1 ; }

doInit() {
  TPLDIR="/mnt/Projects/qube/src/skel.tpl"
  QUBEDIR="build.in"
  [ $DBG -gt 0 ] && OPTS="-v"

  # create the .qube directory
  [ ! -d .qube ] && mkdir .qube
  [ ! -f .qube/config ] && cp "$TPLDIR/config.tpl" .qube/config

  local tSRCDIR="$TPLDIR/src/$QUBEDIR"      # make source dir a bit shorter below
  mkdir -p "$QUBEDIR"
  chmod 3775 "$QUBEDIR"
  for D in control feature layouts sysdeps version ; do
    mkdir -p "$QUBEDIR/$D"
    chmod 3775 "$QUBEDIR/$D"
    for F in `ls "$tSRCDIR/$D"` ; do
      cp -n $OPTS "$tSRCDIR/$D/$F" "$QUBEDIR/$D/"
    done
  done
  # copy all files from build.in
  for F in `ls $tSRCDIR` ; do
    [ -f "$tSRCDIR/$F" ] && cp -n $OPTS "$tSRCDIR/$F" "$QUBEDIR"
  done
  # copy all files in src
  for F in `ls -a "$TPLDIR/src"` ; do
    [ -f "$TPLDIR/src/$F" ] && cp -n $OPTS "$TPLDIR/src/$F" .
  done

  [ ! -f .qube/pkginclude ] && echo "CHANGELOG" > .qube/pkginclude
#  [ ! -f .qube/pkgexclude ] && echo "" > .qube/pkgexclude
  exit 0
}
#***** begin processing *********************************************************

case "$1" in
          B|b*) doBinPkg;;
          E|e*) doEdit $2;;
         L|ch*) doChgLog;;
        N|init) doInit;;
          S|s*) doSrcPkg;;
          U|u*) UpLoad;;
        config) doConfig;;
  -V|--version) version;;
     -h|--help) help;;
       *) echo "Invalid action: $1"; exit 1;;
esac
